@@ -1,14 +1,6 @@
 #![cfg(test)]
 
-use crate::{
-    condition::*,
-    consideration::*,
-    decision_makers::{
-        machinery::*, parallelizer::*, planner::*, reasoner::*, selector::*, sequencer::*,
-    },
-    task::*,
-    Scalar,
-};
+use crate::prelude::*;
 use std::collections::{HashMap, HashSet};
 
 macro_rules! map {
@@ -35,41 +27,41 @@ macro_rules! set {
     };
 }
 
-struct Memory {
-    pub mood: Scalar,
-}
-
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
-enum Mood {
-    Happy,
-    Sad,
-}
-
-#[derive(Debug, Clone)]
-struct MoodStateTask {
-    pub name: String,
-}
-
-impl MoodStateTask {
-    pub fn new(name: impl ToString) -> Self {
-        Self {
-            name: name.to_string(),
-        }
-    }
-}
-
-impl Task<Memory> for MoodStateTask {
-    fn on_enter(&mut self, memory: &mut Memory) {
-        println!("ENTER | state: {} | mood: {}", self.name, memory.mood);
-    }
-
-    fn on_exit(&mut self, memory: &mut Memory) {
-        println!("EXIT | state: {} | mood: {}", self.name, memory.mood);
-    }
-}
-
 #[test]
 fn test_reasoner() {
+    struct Memory {
+        pub mood: Scalar,
+    }
+
+    #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
+    enum Mood {
+        Happy,
+        Sad,
+    }
+
+    #[derive(Debug, Clone)]
+    struct MoodStateTask {
+        pub name: String,
+    }
+
+    impl MoodStateTask {
+        pub fn new(name: impl ToString) -> Self {
+            Self {
+                name: name.to_string(),
+            }
+        }
+    }
+
+    impl Task<Memory> for MoodStateTask {
+        fn on_enter(&mut self, memory: &mut Memory) {
+            println!("ENTER | state: {} | mood: {}", self.name, memory.mood);
+        }
+
+        fn on_exit(&mut self, memory: &mut Memory) {
+            println!("EXIT | state: {} | mood: {}", self.name, memory.mood);
+        }
+    }
+
     #[derive(Debug, Copy, Clone)]
     struct MoodConsideration {
         pub desired: Scalar,
@@ -116,6 +108,39 @@ fn test_reasoner() {
 
 #[test]
 fn test_machinery() {
+    struct Memory {
+        pub mood: Scalar,
+    }
+
+    #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
+    enum Mood {
+        Happy,
+        Sad,
+    }
+
+    #[derive(Debug, Clone)]
+    struct MoodStateTask {
+        pub name: String,
+    }
+
+    impl MoodStateTask {
+        pub fn new(name: impl ToString) -> Self {
+            Self {
+                name: name.to_string(),
+            }
+        }
+    }
+
+    impl Task<Memory> for MoodStateTask {
+        fn on_enter(&mut self, memory: &mut Memory) {
+            println!("ENTER | state: {} | mood: {}", self.name, memory.mood);
+        }
+
+        fn on_exit(&mut self, memory: &mut Memory) {
+            println!("EXIT | state: {} | mood: {}", self.name, memory.mood);
+        }
+    }
+
     #[derive(Debug, Copy, Clone)]
     enum MoodCondition {
         LessThan(Scalar),
@@ -432,3 +457,136 @@ fn test_parallelizer() {
     assert_eq!(parallelizer.process(&mut memory), true);
     assert_eq!(memory, true);
 }
+
+#[test]
+fn test_lod() {
+    const DELTA_TIME: Scalar = 1.0;
+
+    struct Memory {
+        time_since_last_meal: Scalar,
+        hunger: Scalar,
+    }
+
+    let mut lod = Lod::default()
+        // level 0 means agent is not in area and we optimize AI processing by not doing any work,
+        // so on task exit we just estimate how much more hungry agent can get during task time.
+        .level(ClosureTask::default().exit(|m: &mut LodMemory<Memory>| {
+            m.memory.hunger -= m.memory.time_since_last_meal;
+            println!("* Background hunger estimation: {}", m.memory.hunger);
+        }))
+        // level 1 means agent is in area and we want to accurately change its hunger level.
+        .level(ClosureTask::default().update(|m: &mut LodMemory<Memory>| {
+            m.memory.hunger -= DELTA_TIME;
+            println!("* Foreground hunger calculation: {}", m.memory.hunger);
+        }))
+        .build();
+
+    let mut memory = LodMemory {
+        lod_level: 0,
+        memory: Memory {
+            time_since_last_meal: 0.0,
+            hunger: 10.0,
+        },
+    };
+
+    // we start with agent running in the background.
+    assert_eq!(lod.active_state(), None);
+    assert_eq!(lod.process(&mut memory), true);
+    assert_eq!(lod.active_state(), Some(&0));
+    // agent will now run in foreground and we assume 5 seconds have passed since last meal.
+    memory.lod_level = 1;
+    memory.memory.time_since_last_meal = 5.0;
+    assert_eq!(lod.process(&mut memory), true);
+    assert_eq!(lod.active_state(), Some(&1));
+    assert_eq!(memory.memory.hunger, 5.0);
+    lod.update(&mut memory);
+    assert_eq!(memory.memory.hunger, 4.0);
+}
+
+#[test]
+fn test_behavior_tree() {
+    struct Memory {
+        mode: bool,
+        counter: usize,
+    }
+
+    struct Countdown(pub usize);
+
+    impl Task<Memory> for Countdown {
+        fn is_locked(&self, memory: &Memory) -> bool {
+            memory.counter > 0
+        }
+
+        fn on_enter(&mut self, memory: &mut Memory) {
+            memory.counter = self.0;
+        }
+
+        fn on_update(&mut self, memory: &mut Memory) {
+            memory.counter = memory.counter.max(1) - 1;
+        }
+    }
+
+    struct FlipMode;
+
+    impl Task<Memory> for FlipMode {
+        fn on_enter(&mut self, memory: &mut Memory) {
+            memory.mode = !memory.mode;
+        }
+    }
+
+    struct IsMode(pub bool);
+
+    impl Condition<Memory> for IsMode {
+        fn validate(&self, memory: &Memory) -> bool {
+            memory.mode == self.0
+        }
+    }
+
+    // we define a tree that will perform ping-pong with delay:
+    // first we wait 2 turns, flip memory state, then wait 1 turn and flip back memory state.
+    let mut tree = BehaviorTree::selector(true)
+        .node(
+            BehaviorTree::sequence(IsMode(true))
+                .node(BehaviorTree::state(true, Countdown(2)))
+                .node(BehaviorTree::state(true, FlipMode)),
+        )
+        .node(
+            BehaviorTree::sequence(IsMode(false))
+                .node(BehaviorTree::state(true, Countdown(1)))
+                .node(BehaviorTree::state(true, FlipMode)),
+        )
+        .build();
+
+    let mut memory = Memory {
+        mode: true,
+        counter: 0,
+    };
+
+    assert_eq!(tree.on_process(&mut memory), true);
+    assert_eq!(memory.mode, true);
+    assert_eq!(memory.counter, 2);
+
+    assert_eq!(tree.on_process(&mut memory), false);
+    tree.on_update(&mut memory);
+    assert_eq!(memory.mode, true);
+    assert_eq!(memory.counter, 1);
+
+    assert_eq!(tree.on_process(&mut memory), false);
+    tree.on_update(&mut memory);
+    assert_eq!(memory.mode, true);
+    assert_eq!(memory.counter, 0);
+
+    assert_eq!(tree.on_process(&mut memory), true);
+
+    assert_eq!(tree.on_process(&mut memory), true);
+    assert_eq!(memory.mode, false);
+    assert_eq!(memory.counter, 1);
+
+    assert_eq!(tree.on_process(&mut memory), false);
+    tree.on_update(&mut memory);
+    assert_eq!(memory.mode, false);
+    assert_eq!(memory.counter, 0);
+
+    assert_eq!(tree.on_process(&mut memory), true);
+    assert_eq!(memory.mode, true);
+}
